@@ -12,17 +12,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -56,10 +46,10 @@ if typing.TYPE_CHECKING:
     from gajim.common.cert_store import CertificateStore
     from gajim.common.commands import ChatCommands  # noqa: F401
     from gajim.common.preview import PreviewManager
-    from gajim.common.storage.archive import MessageArchiveStorage
+    from gajim.common.storage.archive.storage import MessageArchiveStorage
     from gajim.common.storage.cache import CacheStorage
     from gajim.common.storage.draft import DraftStorage
-    from gajim.common.storage.events import EventStorage
+    from gajim.common.storage.events.storage import EventStorage
     from gajim.common.task_manager import TaskManager
 
     from gajim.gtk.application import GajimApplication  # noqa: F401
@@ -287,7 +277,10 @@ def detect_dependencies() -> None:
     try:
         import sentry_sdk  # noqa: F401
         _dependencies['SENTRY_SDK'] = True
-    except ImportError:
+    except Exception:
+        # Sentry has a lot of side effects on import
+        # make sure this optional dependency does not prevent
+        # Gajim from starting
         pass
 
     # Print results
@@ -380,8 +373,8 @@ def get_accounts_sorted() -> list[str]:
 
 
 def get_enabled_accounts_with_labels(
-        connected_only: bool = False,
-        private_storage_only: bool = False) -> list[list[str]]:
+    connected_only: bool = False
+) -> list[list[str]]:
     '''
     Returns a list with [account, account_label] entries.
     Order by account_label
@@ -390,8 +383,6 @@ def get_enabled_accounts_with_labels(
     for acc in connections:
         if connected_only and not account_is_connected(acc):
             continue
-        if private_storage_only and not account_supports_private_storage(acc):
-            continue
 
         accounts.append([acc, get_account_label(acc)])
 
@@ -403,12 +394,6 @@ def get_account_label(account: str) -> str:
     return settings.get_account_setting(account, 'account_label') or account
 
 
-def account_supports_private_storage(account: str) -> bool:
-    # If Delimiter module is not available we can assume
-    # Private Storage is not available
-    return connections[account].get_module('Delimiter').available
-
-
 def account_is_connected(account: str) -> bool:
     if account not in connections:
         return False
