@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from typing import Any
 from typing import TextIO
@@ -46,26 +36,29 @@ from gajim.common.events import AccountDisabled
 from gajim.common.events import AccountDisconnected
 from gajim.common.events import AccountEnabled
 from gajim.common.events import AllowGajimUpdateCheck
+from gajim.common.events import DBMigrationError
 from gajim.common.events import GajimUpdateAvailable
 from gajim.common.events import SignedIn
 from gajim.common.helpers import from_one_line
 from gajim.common.helpers import get_random_string
 from gajim.common.settings import LegacyConfig
 from gajim.common.settings import Settings
-from gajim.common.storage.archive import MessageArchiveStorage
+from gajim.common.storage.archive.storage import MessageArchiveStorage
 from gajim.common.storage.cache import CacheStorage
 from gajim.common.storage.draft import DraftStorage
-from gajim.common.storage.events import EventStorage
+from gajim.common.storage.events.storage import EventStorage
 from gajim.common.task_manager import TaskManager
 from gajim.common.util.http import create_http_request
 
+log = logging.getLogger('gajim.c.application')
+
 
 class CoreApplication(ged.EventHelper):
     def __init__(self) -> None:
         ged.EventHelper.__init__(self)
         self._profiling_session = None
 
-    def _init_core(self) -> None:
+    def _init_core(self) -> bool:
         # Create and initialize Application Paths & Databases
         app.app = self
         app.print_version()
@@ -80,14 +73,19 @@ class CoreApplication(ged.EventHelper):
         app.config = LegacyConfig()
         app.commands = ChatCommands()
 
-        app.storage.cache = CacheStorage()
-        app.storage.cache.init()
+        try:
+            app.storage.cache = CacheStorage()
+            app.storage.cache.init()
 
-        app.storage.events = EventStorage()
-        app.storage.events.init()
+            app.storage.events = EventStorage()
+            app.storage.events.init()
 
-        app.storage.archive = MessageArchiveStorage()
-        app.storage.archive.init()
+            app.storage.archive = MessageArchiveStorage()
+            app.storage.archive.init()
+        except Exception as error:
+            app.ged.raise_event(DBMigrationError(exception=error))
+            log.exception('Failed to init storage')
+            return False
 
         app.storage.drafts = DraftStorage()
 
@@ -125,6 +123,8 @@ class CoreApplication(ged.EventHelper):
         app.plugin_manager.init_plugins()
         app.plugin_repository = PluginRepository()
 
+        app.commands.init()
+
         for client in app.get_clients():
             client.get_module('Roster').load_roster()
 
@@ -134,6 +134,8 @@ class CoreApplication(ged.EventHelper):
             ('signed-in', ged.CORE, self._on_signed_in),
         ])
 
+        return True
+
     @property
     def _log(self) -> logging.Logger:
         return app.log('gajim.application')
@@ -313,10 +315,10 @@ class CoreApplication(ged.EventHelper):
                      callback=self._on_update_response)
 
     def _on_update_response(self, request: HTTPRequest) -> None:
-
         if not request.is_complete():
-            self._log.warning('Error while performing gajim update check: %s',
-                              request.get_error(), request.get_status())
+            self._log.warning(
+                'Error while performing gajim update check: %s (%s)',
+                request.get_error_string(), request.get_status())
             return
 
         now = datetime.now()
@@ -337,7 +339,7 @@ class CoreApplication(ged.EventHelper):
 
     def create_account(self,
                        account: str,
-                       username: str,
+                       username: str | None,
                        domain: str,
                        password: str,
                        proxy_name: str | None,
@@ -347,10 +349,13 @@ class CoreApplication(ged.EventHelper):
                        anonymous: bool = False
                        ) -> None:
 
-        account_label = f'{username}@{domain}'
         if anonymous:
             username = 'anon'
-            account_label = f'anon@{domain}'
+
+        if not username:
+            raise ValueError('Username must be set')
+
+        account_label = f'{username}@{domain}'
 
         config: dict[str, str | int | bool] = {
             'name': username,
@@ -376,8 +381,9 @@ class CoreApplication(ged.EventHelper):
         for opt, value in config.items():
             app.settings.set_account_setting(account, opt, value)  # pyright: ignore  # noqa: E501
 
-        # Password module depends on existing config
-        passwords.save_password(account, password)
+        if not anonymous:
+            # Password module depends on existing config
+            passwords.save_password(account, password)
 
     def enable_account(self, account: str) -> None:
         app.connections[account] = Client(account)
@@ -415,13 +421,12 @@ class CoreApplication(ged.EventHelper):
         # Delete password must be before del_per() because it calls set_per()
         # which would recreate the account with defaults values if not found
         passwords.delete_password(account)
+        app.storage.archive.remove_account(account)
         app.settings.remove_account(account)
         app.app.remove_account_actions(account)
 
     def _on_signed_in(self, event: SignedIn) -> None:
         client = app.get_client(event.account)
-        app.storage.archive.insert_jid(client.get_own_jid().bare)
-
         if client.get_module('MAM').available:
             client.get_module('MAM').request_archive_on_signin()
 
