@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -27,6 +17,7 @@ from gi.repository import Gtk
 from nbxmpp.client import Client as NBXMPPClient
 from nbxmpp.const import ConnectionType
 from nbxmpp.const import StreamError
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 
 from gajim.common import app
@@ -39,7 +30,6 @@ from gajim.common.const import SimpleClientState
 from gajim.common.events import AccountConnected
 from gajim.common.events import AccountDisconnected
 from gajim.common.events import MessageNotSent
-from gajim.common.events import MessageSent
 from gajim.common.events import Notification
 from gajim.common.events import PasswordRequired
 from gajim.common.events import PlainConnection
@@ -168,6 +158,7 @@ class Client(Observable, ClientModules):
         self._client.set_username(self._user)
         self._client.set_resource(get_resource(self._account))
         self._client.set_http_session(create_http_session())
+        self._client.set_supported_fallback_ns([Namespace.REPLY])
 
         self._client.subscribe('resume-failed', self._on_resume_failed)
         self._client.subscribe('resume-successful', self._on_resume_successful)
@@ -200,6 +191,7 @@ class Client(Observable, ClientModules):
             self.update_presence()
 
         self.notify('state-changed', SimpleClientState.CONNECTED)
+        self.notify('resume-successful')
 
     def _set_client_available(self) -> None:
         self._set_state(ClientState.AVAILABLE)
@@ -266,7 +258,7 @@ class Client(Observable, ClientModules):
 
             if error in ('not-authorized', 'no-password'):
                 def _on_password() -> None:
-                    self._prepare_for_connect()
+                    self.connect()
 
                 app.ged.raise_event(PasswordRequired(client=self,
                                                      on_password=_on_password))
@@ -349,6 +341,12 @@ class Client(Observable, ClientModules):
         # This returns the bare jid
         return nbxmpp.JID.from_string(app.get_jid_from_account(self._account))
 
+    def get_bound_jid(self) -> JID:
+        assert self._client is not None
+        jid = self._client.get_bound_jid()
+        assert jid is not None
+        return jid
+
     def change_status(self, show: str, message: str) -> None:
         if not message:
             message = ''
@@ -369,15 +367,14 @@ class Client(Observable, ClientModules):
                 helpers.to_one_line(message))
 
         if self._state.is_disconnecting:
-            log.warning("Can\'t change status while "
-                        'disconnect is in progress')
+            log.warning("Can't change status while disconnect is in progress")
             return
 
         if self._state.is_disconnected:
             if show == 'offline':
                 return
 
-            self._prepare_for_connect()
+            self.connect()
             return
 
         if self._state.is_connecting:
@@ -392,7 +389,7 @@ class Client(Observable, ClientModules):
                 self._destroy_client = True
                 self._abort_reconnect()
             else:
-                self._prepare_for_connect()
+                self.connect()
             return
 
         # We are connected
@@ -428,10 +425,8 @@ class Client(Observable, ClientModules):
     def connect_machine(self) -> None:
         log.info('Connect machine state: %s', self._connect_machine_calls)
         if self._connect_machine_calls == 1:
-            self.get_module('Delimiter').get_roster_delimiter()
-        elif self._connect_machine_calls == 2:
             self.get_module('Roster').request_roster()
-        elif self._connect_machine_calls == 3:
+        elif self._connect_machine_calls == 2:
             self._finish_connect()
 
     def _finish_connect(self) -> None:
@@ -468,7 +463,7 @@ class Client(Observable, ClientModules):
             return
 
         stanza = self.get_module('Message').build_message_stanza(message)
-        message.stanza = stanza
+        message.set_stanza(stanza)
 
         method = message.contact.settings.get('encryption')
         if not method:
@@ -480,12 +475,15 @@ class Client(Observable, ClientModules):
                 self.get_module('OMEMO').encrypt_message(message)
             except Exception:
                 log.exception('Error')
+                text = message.get_text(with_fallback=False)
+                assert text is not None
                 app.ged.raise_event(
-                    MessageNotSent(client=self._client,
-                                   jid=message.jid,
-                                   message=message.message,
-                                   error=_('Encryption error'),
-                                   time=time.time()))
+                    MessageNotSent(
+                        client=self._client,
+                        jid=str(message.contact.jid),
+                        message=text,
+                        error=_('Encryption error'),
+                        time=time.time()))
                 return
 
             self._send_message(message)
@@ -501,27 +499,21 @@ class Client(Observable, ClientModules):
                                            self._send_message)
 
     def _send_message(self, message: OutgoingMessage) -> None:
-        message.set_sent_timestamp()
-        message.message_id = self.send_stanza(message.stanza)
+        self.send_stanza(message.get_stanza())
+        self.get_module('Message').store_message(message)
 
-        log_line_id = None
-        if not message.is_groupchat:
-            log_line_id = self.get_module('Message').log_message(message)
+    def connect(
+        self,
+        ignored_tls_errors: IgnoredTlsErrorsT = None
+    ) -> None:
 
-        app.ged.raise_event(
-            MessageSent(jid=message.jid,
-                        account=message.account,
-                        message=message.message,
-                        chatstate=message.chatstate,
-                        timestamp=message.timestamp,
-                        additional_data=message.additional_data,
-                        label=message.label,
-                        correct_id=message.correct_id,
-                        message_id=message.message_id,
-                        msg_log_id=log_line_id,
-                        play_sound=message.play_sound))
+        log.info('Connect')
+
+        if self._state not in (ClientState.DISCONNECTED,
+                               ClientState.RECONNECT_SCHEDULED):
+            # Do not try to reco while we are already trying
+            return
 
-    def _prepare_for_connect(self) -> None:
         custom_host = get_custom_host(self._account)
         if custom_host is not None:
             self._client.set_custom_host(*custom_host)
@@ -547,19 +539,10 @@ class Client(Observable, ClientModules):
         password = passwords.get_password(self._account)
         self._client.set_password(password)
 
-        self.connect()
-
-    def connect(self, ignored_tls_errors: IgnoredTlsErrorsT = None) -> None:
-        if self._state not in (ClientState.DISCONNECTED,
-                               ClientState.RECONNECT_SCHEDULED):
-            # Do not try to reco while we are already trying
-            return
-
-        log.info('Connect')
-
         self._client.set_accepted_certificates(
             app.cert_store.get_certificates())
         self._client.set_ignored_tls_errors(ignored_tls_errors)
+
         self._reconnect = True
         self._disable_reconnect_timer()
         self._set_state(ClientState.CONNECTING)
@@ -578,7 +561,7 @@ class Client(Observable, ClientModules):
         self._set_state(ClientState.RECONNECT_SCHEDULED)
         log.info('Reconnect to %s in 3s', self._account)
         self._reconnect_timer_source = GLib.timeout_add_seconds(
-            3, self._prepare_for_connect)
+            3, self.connect)
 
     def _abort_reconnect(self) -> None:
         self._set_state(ClientState.DISCONNECTED)
