@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -18,29 +8,29 @@ import typing
 from typing import Any
 from typing import Union
 
+import datetime
 from collections.abc import Callable
 from dataclasses import dataclass
 from dataclasses import field
+from functools import cached_property
 
 from nbxmpp.const import Affiliation
 from nbxmpp.const import InviteType
 from nbxmpp.const import Role
 from nbxmpp.const import StatusCode
 from nbxmpp.modules.security_labels import Catalog
-from nbxmpp.modules.security_labels import Displaymarking
-from nbxmpp.modules.security_labels import SecurityLabel
 from nbxmpp.protocol import JID
 from nbxmpp.structs import HTTPAuthData
 from nbxmpp.structs import LocationData
-from nbxmpp.structs import ModerationData
 from nbxmpp.structs import RosterItem
 from nbxmpp.structs import TuneData
 
+from gajim.common import app
 from gajim.common.const import EncryptionInfoMsg
 from gajim.common.const import JingleState
-from gajim.common.const import KindConstant
 from gajim.common.file_props import FileProp
-from gajim.common.helpers import AdditionalDataDict
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import MessageType
 
 if typing.TYPE_CHECKING:
     from gajim.common.client import Client
@@ -48,12 +38,11 @@ if typing.TYPE_CHECKING:
 
 ChatListEventT = Union[
     'MessageReceived',
-    'MamMessageReceived',
-    'GcMessageReceived',
-    'MessageUpdated',
+    'MessageCorrected',
     'MessageModerated',
     'PresenceReceived',
     'MessageSent',
+    'MessageDeleted',
     'JingleRequestReceived',
     'FileRequestReceivedEvent'
 ]
@@ -166,15 +155,14 @@ class MessageSent(ApplicationEvent):
     name: str = field(init=False, default='message-sent')
     account: str
     jid: JID
-    message: str
-    message_id: str
-    msg_log_id: int | None
-    chatstate: str | None
-    timestamp: float
-    additional_data: AdditionalDataDict
-    label: SecurityLabel | None
-    correct_id: str | None
-    play_sound: bool
+    pk: int
+    play_sound: bool = False
+
+    @cached_property
+    def message(self) -> mod.Message:
+        m = app.storage.archive.get_message_with_pk(self.pk)
+        assert m is not None
+        return m
 
 
 @dataclass
@@ -187,6 +175,23 @@ class MessageNotSent(ApplicationEvent):
     time: float
 
 
+@dataclass
+class MessageDeleted(ApplicationEvent):
+    name: str = field(init=False, default='message-deleted')
+    account: str
+    jid: JID
+    pk: int
+
+
+@dataclass
+class MessageAcknowledged(ApplicationEvent):
+    name: str = field(init=False, default='message-acknowledged')
+    account: str
+    jid: JID
+    pk: int
+    stanza_id: str | None
+
+
 @dataclass
 class FileProgress(ApplicationEvent):
     name: str = field(init=False, default='file-progress')
@@ -303,6 +308,14 @@ class DisplayedReceived(ApplicationEvent):
     marker_id: str
 
 
+@dataclass
+class ReactionUpdated(ApplicationEvent):
+    name: str = field(init=False, default='reaction-updated')
+    account: str
+    jid: JID
+    reaction_id: str
+
+
 @dataclass
 class HttpAuth(ApplicationEvent):
     name: str = field(init=False, default='http-auth')
@@ -368,14 +381,18 @@ class ArchivingIntervalFinished(ApplicationEvent):
 
 
 @dataclass
-class MessageUpdated(ApplicationEvent):
-    name: str = field(init=False, default='message-updated')
+class MessageCorrected(ApplicationEvent):
+    name: str = field(init=False, default='message-corrected')
     account: str
     jid: JID
-    msgtxt: str
-    nickname: str | None
-    properties: Any
-    correct_id: str
+    pk: int
+    correction_id: str
+
+    @cached_property
+    def message(self) -> mod.Message:
+        m = app.storage.archive.get_message_with_pk(self.pk)
+        assert m is not None
+        return m
 
 
 @dataclass
@@ -383,52 +400,23 @@ class MessageModerated(ApplicationEvent):
     name: str = field(init=False, default='message-moderated')
     account: str
     jid: JID
-    moderation: ModerationData
-
-
-@dataclass
-class MamMessageReceived(ApplicationEvent):
-    name: str = field(init=False, default='mam-message-received')
-    account: str
-    jid: JID
-    msgtxt: str
-    properties: Any
-    additional_data: AdditionalDataDict
-    unique_id: str
-    stanza_id: str
-    archive_jid: str
-    kind: KindConstant
-    occupant_id: str | None
-    real_jid: JID | None
-    msg_log_id: int | None
-    displaymarking: Displaymarking | None
+    moderation: mod.Moderation
 
 
 @dataclass
 class MessageReceived(ApplicationEvent):
     name: str = field(init=False, default='message-received')
-    conn: 'Client'
-    stanza: Any
     account: str
     jid: JID
-    msgtxt: str
-    properties: Any
-    additional_data: AdditionalDataDict
-    unique_id: str
-    stanza_id: str
-    fjid: str
-    resource: str | None
-    delayed: float | None
-    msg_log_id: int | None
-    displaymarking: Displaymarking | None
+    m_type: MessageType
+    from_mam: bool
+    pk: int
 
-
-@dataclass
-class GcMessageReceived(MessageReceived):
-    name: str = field(init=False, default='gc-message-received')
-    room_jid: str
-    real_jid: JID | None
-    occupant_id: str | None
+    @cached_property
+    def message(self) -> mod.Message:
+        m = app.storage.archive.get_message_with_pk(self.pk)
+        assert m is not None
+        return m
 
 
 @dataclass
@@ -436,7 +424,6 @@ class MessageError(ApplicationEvent):
     name: str = field(init=False, default='message-error')
     account: str
     jid: JID
-    room_jid: str
     message_id: str
     error: Any
 
@@ -699,33 +686,33 @@ class MUCNicknameChanged(ApplicationEvent):
     is_self: bool
     new_name: str
     old_name: str
-    timestamp: float
+    timestamp: datetime.datetime
 
 
 @dataclass
 class MUCRoomConfigChanged(ApplicationEvent):
     name: str = field(init=False, default='muc-room-config-changed')
-    timestamp: float
+    timestamp: datetime.datetime
     status_codes: set[StatusCode]
 
 
 @dataclass
 class MUCRoomConfigFinished(ApplicationEvent):
     name: str = field(init=False, default='muc-room-config-finished')
-    timestamp: float
+    timestamp: datetime.datetime
 
 
 @dataclass
 class MUCRoomPresenceError(ApplicationEvent):
     name: str = field(init=False, default='muc-room-presence-error')
-    timestamp: float
+    timestamp: datetime.datetime
     error: str
 
 
 @dataclass
 class MUCRoomKicked(ApplicationEvent):
     name: str = field(init=False, default='muc-room-kicked')
-    timestamp: float
+    timestamp: datetime.datetime
     status_codes: set[StatusCode] | None
     reason: str | None
     actor: str | None
@@ -734,7 +721,7 @@ class MUCRoomKicked(ApplicationEvent):
 @dataclass
 class MUCRoomDestroyed(ApplicationEvent):
     name: str = field(init=False, default='muc-room-destroyed')
-    timestamp: float
+    timestamp: datetime.datetime
     reason: str | None
     alternate: JID | None
 
@@ -742,7 +729,7 @@ class MUCRoomDestroyed(ApplicationEvent):
 @dataclass
 class MUCUserJoined(ApplicationEvent):
     name: str = field(init=False, default='muc-user-joined')
-    timestamp: float
+    timestamp: datetime.datetime
     is_self: bool
     nick: str
     status_codes: set[StatusCode] | None
@@ -751,7 +738,7 @@ class MUCUserJoined(ApplicationEvent):
 @dataclass
 class MUCUserLeft(ApplicationEvent):
     name: str = field(init=False, default='muc-user-left')
-    timestamp: float
+    timestamp: datetime.datetime
     is_self: bool
     nick: str
     status_codes: set[StatusCode] | None
@@ -762,7 +749,7 @@ class MUCUserLeft(ApplicationEvent):
 @dataclass
 class MUCUserRoleChanged(ApplicationEvent):
     name: str = field(init=False, default='muc-user-role-changed')
-    timestamp: float
+    timestamp: datetime.datetime
     is_self: bool
     nick: str
     role: Role
@@ -773,7 +760,7 @@ class MUCUserRoleChanged(ApplicationEvent):
 @dataclass
 class MUCUserAffiliationChanged(ApplicationEvent):
     name: str = field(init=False, default='muc-user-affiliation-changed')
-    timestamp: float
+    timestamp: datetime.datetime
     is_self: bool
     nick: str
     affiliation: Affiliation
@@ -781,10 +768,18 @@ class MUCUserAffiliationChanged(ApplicationEvent):
     actor: str | None
 
 
+@dataclass
+class MUCAffiliationChanged(ApplicationEvent):
+    name: str = field(init=False, default='room-affiliation-changed')
+    timestamp: datetime.datetime
+    nick: str
+    affiliation: Affiliation
+
+
 @dataclass
 class MUCUserStatusShowChanged(ApplicationEvent):
     name: str = field(init=False, default='muc-user-status-show-changed')
-    timestamp: float
+    timestamp: datetime.datetime
     is_self: bool
     nick: str
     status: str
@@ -797,3 +792,32 @@ class EncryptionInfo(ApplicationEvent):
     account: str
     jid: JID
     message: EncryptionInfoMsg
+
+
+@dataclass
+class DBMigration(ApplicationEvent):
+    name: str = field(init=False, default='db-migration')
+
+
+@dataclass
+class DBMigrationProgress(ApplicationEvent):
+    name: str = field(init=False, default='db-migration-progress')
+    count: int
+    progress: int
+
+    @property
+    def value(self) -> str:
+        value = 100
+        if self.count != 0:
+            value = self.progress / self.count * 100
+        return f'{value:05.2f}'
+
+
+@dataclass
+class DBMigrationFinished(ApplicationEvent):
+    name: str = field(init=False, default='db-migration-finished')
+
+@dataclass
+class DBMigrationError(ApplicationEvent):
+    name: str = field(init=False, default='db-migration-error')
+    exception: Exception
