@@ -1,18 +1,8 @@
 # Copyright (C) 2019 Philipp Hörist <philipp AT hoerist.com>
 #
-# This file is part of OMEMO Gajim Plugin.
+# This file is part of Gajim.
 #
-# OMEMO Gajim Plugin is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# OMEMO Gajim Plugin is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OMEMO Gajim Plugin. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0384: OMEMO Encryption
 
@@ -37,6 +27,7 @@ from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.protocol import NodeProcessed
 from nbxmpp.protocol import Presence
+from nbxmpp.structs import EncryptionData
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import OMEMOMessage
 from nbxmpp.structs import PresenceProperties
@@ -58,7 +49,6 @@ from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import ged
 from gajim.common import types
-from gajim.common.const import EncryptionData
 from gajim.common.const import EncryptionInfoMsg
 from gajim.common.const import Trust as GajimTrust
 from gajim.common.const import XmppUriQuery
@@ -66,6 +56,7 @@ from gajim.common.events import EncryptionInfo
 from gajim.common.events import MucAdded
 from gajim.common.events import MucDiscoUpdate
 from gajim.common.events import SignedIn
+from gajim.common.helpers import event_filter
 from gajim.common.i18n import _
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.contacts import GroupchatContact
@@ -91,7 +82,10 @@ ALLOWED_TAGS = [
     ('no-permanent-store', Namespace.HINTS),
     ('replace', Namespace.CORRECT),
     ('thread', None),
+    ('reply', Namespace.REPLY),
+    ('fallback', Namespace.FALLBACK),
     ('origin-id', Namespace.SID),
+    ('reactions', Namespace.REACTIONS),
 ]
 
 DeviceIdT = int
@@ -150,14 +144,17 @@ class OMEMO(BaseModule):
         self._omemo_groupchats: set[str] = set()
         self._muc_temp_store: dict[bytes, str] = {}
 
+    @event_filter(['account'])
     def _on_signed_in(self, _event: SignedIn) -> None:
         self._log.info('Publish our bundle after sign in')
         self.set_bundle()
         self.request_devicelist()
 
+    @event_filter(['account'])
     def _on_muc_disco_update(self, event: MucDiscoUpdate) -> None:
         self._check_if_omemo_capable(str(event.jid))
 
+    @event_filter(['account'])
     def _on_muc_added(self, event: MucAdded) -> None:
         client = app.get_client(event.account)
         contact = client.get_module('Contacts').get_contact(event.jid)
@@ -269,31 +266,38 @@ class OMEMO(BaseModule):
     def _is_omemo_groupchat(self, room_jid: str) -> bool:
         return room_jid in self._omemo_groupchats
 
-    def encrypt_message(self, event: OutgoingMessage) -> bool:
-        if not event.message:
+    def encrypt_message(self, message: OutgoingMessage) -> bool:
+        if not message.has_text():
             return False
 
-        client = app.get_client(self._account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
+        remote_jid = message.contact.jid
 
-        omemo_message = self.backend.encrypt(str(event.jid),
-                                             event.message,
+        client = app.get_client(self._account)
+        contact = client.get_module('Contacts').get_contact(remote_jid)
+
+        text = message.get_text()
+        assert text is not None
+        omemo_message = self.backend.encrypt(str(remote_jid),
+                                             text,
                                              groupchat=contact.is_groupchat)
         if omemo_message is None:
             raise Exception('Encryption error')
 
-        create_omemo_message(event.stanza, omemo_message,
+        create_omemo_message(message.get_stanza(), omemo_message,
                              node_whitelist=ALLOWED_TAGS)
 
-        if event.is_groupchat:
-            self._muc_temp_store[omemo_message.payload] = event.message
-        else:
-            event.xhtml = None
-            event.additional_data['encrypted'] = {
-                'name': 'OMEMO',
-                'trust': GajimTrust[OMEMOTrust.VERIFIED.name]}
+        if message.is_groupchat:
+            self._muc_temp_store[omemo_message.payload] = text
 
-        self._debug_print_stanza(event.stanza)
+        message.set_encryption(
+            EncryptionData(
+                protocol='OMEMO',
+                key='Unknown',
+                trust=GajimTrust[OMEMOTrust.VERIFIED.name]
+            )
+        )
+
+        self._debug_print_stanza(message.get_stanza())
         return True
 
     def encrypt_file(self,
@@ -378,10 +382,8 @@ class OMEMO(BaseModule):
                 self._log.warning("Can't decrypt own group chat message")
                 return
 
-            plaintext = self._muc_temp_store[properties.omemo.payload]
-            fingerprint = self.backend.get_our_fingerprint()
-            trust = OMEMOTrust.VERIFIED
             del self._muc_temp_store[properties.omemo.payload]
+            return
 
         except DecryptionFailed:
             return
@@ -400,10 +402,11 @@ class OMEMO(BaseModule):
 
         prepare_stanza(stanza, plaintext)
         self._debug_print_stanza(stanza)
-        properties.encrypted = EncryptionData({
-            'name': 'OMEMO',
-            'fingerprint': fingerprint,
-            'trust': GajimTrust[trust.name]})
+        properties.encrypted = EncryptionData(
+            protocol='OMEMO',
+            key=fingerprint or 'Unknown',
+            trust=GajimTrust[trust.name]
+        )
 
     def _process_muc_message(self,
                              properties: MessageProperties
