@@ -1,20 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
+from typing import Any
 from typing import Literal
 from typing import TypedDict
-from typing import Union
 
 import uuid
 
@@ -75,6 +65,7 @@ BoolSettings = Literal[
     'preview_verify_https',
     'print_status_in_chats',
     'remote_control',
+    'show_main_menu',
     'save_main_window_position',
     'send_on_ctrl_enter',
     'show_chatstate_in_banner',
@@ -84,6 +75,7 @@ BoolSettings = Literal[
     'show_notifications_away',
     'show_only_chat_and_online',
     'show_send_message_button',
+    'show_voice_message_button',
     'show_status_msgs_in_roster',
     'show_subject_on_join',
     'show_transports_group',
@@ -126,6 +118,10 @@ IntSettings = Literal[
     'preview_size',
 ]
 
+FloatSettings = Literal[
+    'app_font_size',
+]
+
 StringSettings = Literal[
     'action_on_close',
     'additional_uri_schemes',
@@ -160,10 +156,10 @@ StringSettings = Literal[
     'video_size',
 ]
 
-AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool, list[str]]
+AllSettings = Literal[BoolSettings, IntSettings, FloatSettings, StringSettings]
+AllSettingsT = str | int | float | bool | list[str]
 
-APP_SETTINGS = {
+APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'additional_uri_schemes': '',
     'allow_open_file_uris': False,
     'always_english_wikipedia': False,
@@ -208,7 +204,7 @@ APP_SETTINGS = {
     'gc_sync_threshold_private_default': 0,
     'gc_sync_threshold_public_default': 1,
     'global_proxy': '',
-    'groupchat_roster_width': 210,
+    'groupchat_roster_width': 250,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
     'is_window_visible': True,
@@ -253,11 +249,13 @@ APP_SETTINGS = {
     'show_chatstate_in_banner': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
+    'show_main_menu': True,
     'show_main_window_on_startup': 'always',
     'show_notifications': True,
     'show_notifications_away': False,
     'show_only_chat_and_online': False,
     'show_send_message_button': False,
+    'show_voice_message_button': True,
     'show_status_msgs_in_roster': True,
     'show_subject_on_join': True,
     'show_transports_group': True,
@@ -275,9 +273,10 @@ APP_SETTINGS = {
     'use_keyring': True,
     'use_kib_mib': False,
     'use_libappindicator': True,
-    'use_speller': False,
+    'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
+    'app_font_size': 1.0,
     'video_framerate': '',
     'video_input_device': 'autovideosrc',
     'video_see_self': True,
@@ -314,7 +313,8 @@ BoolAccountSettings = Literal[
     'test_ft_proxies_on_startup',
     'use_custom_host',
     'use_ft_proxies',
-    'use_plain_connection'
+    'use_plain_connection',
+    'omemo_blind_trust'
 ]
 
 
@@ -375,6 +375,7 @@ BoolGroupChatSettings = Literal[
 
 StringGroupChatSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -388,7 +389,7 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
                                IntGroupChatSettings,
                                StringGroupChatSettings]
 
-AllGroupChatSettingsT = Union[str, int, bool]
+AllGroupChatSettingsT = str | int | bool
 
 
 BoolContactSettings = Literal[
@@ -397,6 +398,7 @@ BoolContactSettings = Literal[
 
 StringContactSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -404,7 +406,7 @@ StringContactSettings = Literal[
 AllContactSettings = Literal[BoolContactSettings,
                              StringContactSettings]
 
-AllContactSettingsT = Union[str, bool]
+AllContactSettingsT = str | bool
 
 
 ACCOUNT_SETTINGS = {
@@ -434,6 +436,7 @@ ACCOUNT_SETTINGS = {
         'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
+        'encryption_default': '',
         'file_transfer_proxies': '',
         'filetransfer_preference': 'httpupload',
         'ft_send_local_ips': True,
@@ -474,17 +477,20 @@ ACCOUNT_SETTINGS = {
         'zeroconf_first_name': '',
         'zeroconf_jabber_id': '',
         'zeroconf_last_name': '',
+        'omemo_blind_trust': True,
     },
 
     'contact': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
+        'mute_until': '',
         'send_chatstate': HAS_ACCOUNT_DEFAULT,
         'send_marker': HAS_ACCOUNT_DEFAULT,
         'speller_language': '',
     },
 
     'group_chat': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
+        'mute_until': '',
         'notify_on_all_messages': HAS_APP_DEFAULT,
         'print_join_left': HAS_APP_DEFAULT,
         'print_status': HAS_APP_DEFAULT,
@@ -513,7 +519,7 @@ class OpenChatSettingDetails(TypedDict):
 
 
 OpenChatsSettingT = list[OpenChatSettingDetails]
-AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
+AllWorkspaceSettingsT = str | OpenChatsSettingT
 
 
 class WorkspaceSettings(TypedDict):
@@ -665,7 +671,6 @@ ADVANCED_SETTINGS = {
             'account in the Accounts window. BE CAREFUL, when you are logged '
             'in with a negative priority, you will NOT receive any message '
             'from your server.'),
-        'escape_key_closes': _('Close a chat by pressing the Escape key'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
