@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from typing import Any
 from typing import cast
@@ -52,6 +42,7 @@ from gajim.common.setting_values import BoolContactSettings
 from gajim.common.setting_values import BoolGroupChatSettings
 from gajim.common.setting_values import BoolSettings
 from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
+from gajim.common.setting_values import FloatSettings
 from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
 from gajim.common.setting_values import HAS_APP_DEFAULT
 from gajim.common.setting_values import INITAL_WORKSPACE
@@ -100,7 +91,7 @@ CREATE_SQL = '''
     INSERT INTO settings(name, settings) VALUES ('workspaces', '{workspaces}');
 
     PRAGMA user_version={version};
-    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),
+    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),  # noqa: UP032
                proxies=json.dumps(PROXY_EXAMPLES),
                workspaces=json.dumps(INITAL_WORKSPACE),
                version=CURRENT_USER_VERSION)
@@ -153,7 +144,7 @@ class Settings:
         self._callbacks[(setting, account, jid)].append(weak_func)
 
     def disconnect_signals(self, object_: object) -> Any:
-        for _, handlers in self._callbacks.items():
+        for handlers in self._callbacks.values():
             for handler in list(handlers):
                 if isinstance(handler, tuple):
                     continue
@@ -239,6 +230,9 @@ class Settings:
         if IS_PORTABLE:
             APP_SETTINGS['use_keyring'] = False
 
+        if sys.platform == 'win32':
+            APP_SETTINGS['app_font_size'] = 1.125
+
     def _load_app_overrides(self) -> None:
         if not OVERRIDES_PATH.exists():
             return
@@ -440,12 +434,12 @@ class Settings:
         # Migrate deprecated settings
         value = app_settings.pop('send_chatstate_muc_default', None)
         if value is not None:
-            for _account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['gc_send_chatstate_default'] = value
 
         value = app_settings.pop('send_chatstate_default', None)
         if value is not None:
-            for _account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['send_chatstate_default'] = value
 
         value = app_settings.pop('print_join_left_default', None)
@@ -628,6 +622,10 @@ class Settings:
     def get_app_setting(self, setting: IntSettings) -> int:
         ...
 
+    @overload
+    def get_app_setting(self, setting: FloatSettings) -> float:
+        ...
+
     def get_app_setting(self, setting: str) -> AllSettingsT:
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
@@ -657,6 +655,12 @@ class Settings:
                         value: int | None) -> None:
         ...
 
+    @overload
+    def set_app_setting(self,
+                        setting: FloatSettings,
+                        value: float | None) -> None:
+        ...
+
     @overload
     def set_app_setting(self,
                         setting: Literal['workspace_order'],
@@ -740,6 +744,9 @@ class Settings:
 
     def add_account(self, account: str) -> None:
         log.info('Add account: %s', account)
+        if account in self._account_settings:
+            raise ValueError('Account %s exists already' % account)
+
         self._account_settings[account] = {'account': {},
                                            'contact': {},
                                            'group_chat': {}}
@@ -761,6 +768,14 @@ class Settings:
     def get_accounts(self) -> list[str]:
         return list(self._account_settings.keys())
 
+    def account_exists(self, jid: str) -> bool:
+        for account in self._account_settings:
+            name = self.get_account_setting(account, 'name')
+            hostname = self.get_account_setting(account, 'hostname')
+            if jid == f'{name}@{hostname}':
+                return True
+        return False
+
     def get_active_accounts(self) -> list[str]:
         active: list[str] = []
         for account in self._account_settings:
@@ -768,6 +783,14 @@ class Settings:
                 active.append(account)
         return active
 
+    def get_account_from_jid(self, jid: JID) -> str:
+        for account in self._account_settings:
+            name = self.get_account_setting(account, 'name')
+            hostname = self.get_account_setting(account, 'hostname')
+            if jid.localpart == name and jid.domain == hostname:
+                return account
+        raise ValueError(f'No account found for: {jid}')
+
     @overload
     def get_account_setting(self,
                             account: str,
@@ -910,12 +933,18 @@ class Settings:
                 return self.get_app_setting(f'gc_{setting}_default')
 
             if default is HAS_ACCOUNT_DEFAULT:
-                context_default_setting = f'gc_{setting}_{context}_default'
-                if context_default_setting in ACCOUNT_SETTINGS['account']:
-                    return self.get_account_setting(account,
-                                                    context_default_setting)
-                return self.get_account_setting(account,
-                                                f'gc_{setting}_default')
+                default_settings = [
+                    f'gc_{setting}_{context}_default',
+                    f'gc_{setting}_default',
+                    f'{setting}_default',
+                ]
+
+                for default_setting in default_settings:
+                    if default_setting in ACCOUNT_SETTINGS['account']:
+                        return self.get_account_setting(
+                            account, default_setting)
+
+                raise ValueError(f'No default setting found for {setting}')
 
             return default
 
@@ -972,9 +1001,20 @@ class Settings:
             if default is HAS_ACCOUNT_DEFAULT:
                 default_store = ACCOUNT_SETTINGS['account']
 
-            default = default_store.get(f'gc_{setting}_{context}_default')
+            default_settings = [
+                f'gc_{setting}_{context}_default',
+                f'gc_{setting}_default',
+                f'{setting}_default',
+            ]
+
+            default = None
+            for default_setting in default_settings:
+                if default_setting in default_store:
+                    default = default_store.get(default_setting)
+                    break
+
             if default is None:
-                default = default_store.get(f'gc_{setting}_default')
+                raise ValueError(f'No default setting found for {setting}')
 
         if not isinstance(value, type(default)) and value is not None:
             raise TypeError(f'Invalid type for {setting}: '
