@@ -21,17 +21,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -53,7 +43,6 @@ from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import ConnectionType
-from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import InvalidJid
 
 import gajim
@@ -87,6 +76,7 @@ from gajim.gtk.const import MuteState
 from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ErrorDialog
 from gajim.gtk.dialogs import ShortcutsWindow
 from gajim.gtk.discovery import ServiceDiscoveryWindow
 from gajim.gtk.start_chat import StartChatDialog
@@ -224,8 +214,13 @@ class GajimApplication(Gtk.Application, CoreApplication):
             # to render colored emoji glyphs
             os.environ['PANGOCAIRO_BACKEND'] = 'fontconfig'
 
-        self._init_core()
+        app.ged.register_event_handler(
+            'db-migration', 0, self._on_db_migration)
 
+        if not self._init_core():
+            return
+
+        Gtk.IconSize.register('100', 100, 100)
         icon_theme = Gtk.IconTheme.get_default()
         icon_theme.append_search_path(str(configpaths.get('ICONS')))
         load_user_iconsets()
@@ -490,10 +485,20 @@ class GajimApplication(Gtk.Application, CoreApplication):
         for action_name in ONLINE_ACCOUNT_ACTIONS:
             self.set_action_state(f'{account}-{action_name}', new_state)
 
+        # Disable all feature actions on disconnect
         if not new_state:
             for action_name in FEATURE_ACCOUNT_ACTIONS:
                 self.set_action_state(f'{account}-{action_name}', new_state)
 
+    def update_feature_actions_state(self, account: str) -> None:
+        client = app.get_client(account)
+        mam_available = client.get_module('MAM').available
+        blocking_available = client.get_module('Blocking').supported
+
+        self.set_action_state(f'{account}-archive', mam_available)
+        self.set_action_state(f'{account}-blocking', blocking_available)
+        self.set_action_state(f'{account}-block-contact', blocking_available)
+
     def update_app_actions_state(self) -> None:
         active_accounts = bool(app.get_connected_accounts(exclude_local=True))
         self.set_action_state('create-groupchat', active_accounts)
@@ -518,14 +523,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             self.set_accels_for_action(action, accels)
 
     def _on_feature_discovered(self, event: events.FeatureDiscovered) -> None:
-        if event.feature == Namespace.MAM_2:
-            action = '%s-archive' % event.account
-            self.set_action_state(action, True)
-        elif event.feature == Namespace.BLOCKING:
-            action = '%s-blocking' % event.account
-            self.set_action_state(action, True)
-            action = '%s-block-contact' % event.account
-            self.set_action_state(action, True)
+        self.update_feature_actions_state(event.account)
 
     def create_account(self,
                        account: str,
@@ -587,6 +585,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
         if window is not None:
             window.remove_account(account)
 
+    def _on_db_migration(self, _event: events.DBMigration) -> None:
+        open_window('DBMigration')
+
     # Action Callbacks
 
     @staticmethod
@@ -656,8 +657,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
             try:
                 # Object will add itself to the window dict
                 ServiceDiscoveryWindow(account, address_entry=True)
-            except GajimGeneralException:
-                pass
+            except (GajimGeneralException, RuntimeError) as e:
+                ErrorDialog(_('Error'), str(e))
 
     @staticmethod
     def _on_create_groupchat_action(_action: Gio.SimpleAction,
@@ -722,7 +723,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
     @staticmethod
     def _on_xml_console_action(_action: Gio.SimpleAction,
                                _param: GLib.Variant | None) -> None:
-        open_window('XMLConsoleWindow')
+        open_window('DebugConsoleWindow')
 
     @staticmethod
     def _on_manage_proxies_action(_action: Gio.SimpleAction,
@@ -851,7 +852,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
                                   params: structs.RemoveHistoryActionParams
                                   ) -> None:
         def _remove() -> None:
-            app.storage.archive.remove_history(params.account, params.jid)
+            app.storage.archive.remove_history_for_jid(
+                params.account, params.jid)
 
             app.window.clear_chat_list_row(params.account, params.jid)
             control = app.window.get_control()
@@ -882,4 +884,4 @@ class GajimApplication(Gtk.Application, CoreApplication):
         client.get_module('MUC').leave(params.jid)
         client.get_module('Bookmarks').remove(params.jid)
 
-        app.storage.archive.remove_history(params.account, params.jid)
+        app.storage.archive.remove_history_for_jid(params.account, params.jid)
