@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -243,17 +233,20 @@ def get_show_circle(show: str | types.PresenceShowT,
     if not isinstance(show, str):
         show = show.value
 
-    size = size * scale
-    center = size / 2
-    radius = size / 3
+    width = size * scale
+    height = width
 
-    surface = cairo.ImageSurface(cairo.Format.ARGB32, size, size)
+    surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+    surface.set_device_scale(scale, scale)
     context = cairo.Context(surface)
 
     css_color = get_css_show_class(show)
     color = convert_rgb_string_to_float(
         app.css_config.get_value(css_color, StyleAttr.COLOR))
 
+    center = size / 2
+    radius = size / 3
+
     context.set_source_rgb(*color)
     context.set_operator(cairo.Operator.OVER)
     context.arc(center, center, radius, 0, 2 * pi)
@@ -479,11 +472,13 @@ class AvatarStorage(metaclass=Singleton):
             if avatar_sha is not None:
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
+                    # clip first to avoid clipping the transport icon
+                    surface = clip(surface, style)
+
                     if transport_icon is not None:
                         surface = add_transport_to_avatar(
                             surface, transport_icon)
 
-                    surface = clip(surface, style)
                     self._cache[jid][
                         (size, scale, None, transport_icon)] = surface
                     return surface
@@ -519,7 +514,7 @@ class AvatarStorage(metaclass=Singleton):
         avatar_sha = app.settings.get_workspace_setting(
             workspace_id, 'avatar_sha')
         if avatar_sha:
-            surface = self._load_surface_from_storage(avatar_sha, size, scale)
+            surface = self.surface_from_filename(avatar_sha, size, scale)
             if surface is not None:
                 return clip(surface, 'round-corners')
 
@@ -598,22 +593,6 @@ class AvatarStorage(metaclass=Singleton):
         surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale)
         return fit(surface, size)
 
-    def _load_surface_from_storage(self,
-                                   filename: str,
-                                   size: int,
-                                   scale: int) -> cairo.ImageSurface | None:
-
-        size = size * scale
-        path = self.get_avatar_path(filename)
-        if path is None:
-            return None
-
-        pixbuf = load_pixbuf(path, size)
-        if pixbuf is None:
-            return None
-        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale)
-        return fit(surface, size)
-
     def _get_avatar_from_storage(self,
                                  contact: (types.BareContact |
                                            types.GroupchatContact |
@@ -626,7 +605,7 @@ class AvatarStorage(metaclass=Singleton):
         if avatar_sha is None:
             return None
 
-        surface = self._load_surface_from_storage(avatar_sha, size, scale)
+        surface = self.surface_from_filename(avatar_sha, size, scale)
         if surface is None:
             return None
         return clip(surface, style)
