@@ -1,22 +1,11 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import logging
 from pathlib import Path
@@ -58,7 +47,7 @@ class FileTransferSelector(Gtk.Box):
 
     def __init__(self,
                  contact: types.ChatContactT,
-                 method: Optional[str] = None
+                 method: str | None = None
                  ) -> None:
 
         Gtk.Box.__init__(self)
@@ -113,6 +102,11 @@ class FileTransferSelector(Gtk.Box):
     def _is_over_max_http_file_size(self, path: Path) -> bool:
         if self._max_http_file_size is None:
             return False
+
+        if not path.exists():
+            # Non-existing files are handled in the send_file method
+            return False
+
         return path.stat().st_size > self._max_http_file_size
 
     def transfer_resource_required(self) -> bool:
@@ -288,8 +282,8 @@ class FileRow(Gtk.ListBoxRow):
         load_file_async(self.file_path, self._on_load_finished, mime_type)
 
     def _on_load_finished(self,
-                          data: Optional[bytes],
-                          _error: Optional[GLib.Error],
+                          data: bytes | None,
+                          _error: GLib.Error | None,
                           mime_type: Any
                           ) -> None:
 
