@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -46,6 +36,7 @@ from gajim.common.modules.bytestream import is_transfer_active
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.storage.archive.const import MessageType
 from gajim.plugins.manifest import PluginManifest
 from gajim.plugins.repository import PluginRepository
 
@@ -68,7 +59,8 @@ from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import actionmethod
 from gajim.gtk.structs import AddChatActionParams
 from gajim.gtk.structs import ChatListEntryParam
-from gajim.gtk.structs import RetractMessageParam
+from gajim.gtk.structs import DeleteMessageParam
+from gajim.gtk.structs import ModerateMessageParam
 from gajim.gtk.util import get_app_window
 from gajim.gtk.util import open_window
 from gajim.gtk.util import resize_window
@@ -122,6 +114,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.connect('notify::is-active', self._on_window_active)
         self.connect('delete-event', self._on_window_delete)
         self.connect('window-state-event', self._on_window_state_changed)
+        self.connect_after('key-press-event', self._on_key_press_event)
 
         self._ui.connect_signals(self)
 
@@ -160,9 +153,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.app.systray.connect_unread_widget(chat_list_stack,
                                               'unread-count-changed')
 
+        self.set_show_menubar(app.settings.get_app_setting('show_main_menu'))
+
         for client in app.get_clients():
             client.connect_signal('state-changed',
                                   self._on_client_state_changed)
+            client.connect_signal('resume-successful',
+                                  self._on_client_resume_successful)
 
     def get_action(self, name: str) -> Gio.SimpleAction:
         action = self.lookup_action(name)
@@ -239,6 +236,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._main_stack.add_account_page(event.account)
         client = app.get_client(event.account)
         client.connect_signal('state-changed', self._on_client_state_changed)
+        client.connect_signal('resume-successful',
+                              self._on_client_resume_successful)
 
     def _on_account_disabled(self, event: events.AccountDisabled) -> None:
         workspace_id = self._workspace_side_bar.get_first_workspace()
@@ -250,6 +249,47 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def update_account_unread_count(self, account: str, count: int) -> None:
         self._account_side_bar.update_unread_count(account, count)
 
+    def _on_key_press_event(
+        self,
+        _window: MainWindow,
+        event: Gdk.EventKey
+    ) -> bool:
+
+        # event.get_state() behaves different on Linux and Windows.
+        # On Linux its not set in the case that only a modifier key
+        # is pressed.
+
+        # Filter out modifier not used for shortcuts like Numlock (MOD2)
+        modifier = event.get_state() & Gtk.accelerator_get_default_mod_mask()
+        accel_name = Gtk.accelerator_name(event.keyval, modifier)
+
+        log.info('Captured key pressed: %s', accel_name)
+
+        if event.keyval in (
+            Gdk.KEY_Control_L,
+            Gdk.KEY_Control_R,
+            Gdk.KEY_Alt_L,
+            Gdk.KEY_Alt_R,
+            Gdk.KEY_Shift_L,
+            Gdk.KEY_Shift_R,
+        ):
+            return False
+
+        focused_widget = self.get_focus()
+        if (isinstance(focused_widget, Gtk.TextView)
+            and focused_widget.props.editable):
+                return False
+
+        if isinstance(focused_widget, Gtk.Entry):
+            return False
+
+        message_input = self.get_chat_stack().get_message_input()
+        if not message_input.get_mapped() or not message_input.is_sensitive():
+            return False
+
+        message_input.grab_focus()
+        return self.propagate_key_event(event)
+
     def _on_client_state_changed(self,
                                  client: Client,
                                  _signal_name: str,
@@ -258,6 +298,12 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.app.set_account_actions_state(client.account, state.is_connected)
         app.app.update_app_actions_state()
 
+    def _on_client_resume_successful(self,
+                                     client: Client,
+                                     _signal_name: str) -> None:
+
+        app.app.update_feature_actions_state(client.account)
+
     def _on_allow_gajim_update(self,
                                event: events.AllowGajimUpdateCheck) -> None:
         self.add_app_message(event.name)
@@ -420,6 +466,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('decrease-app-font-size', self._on_app_font_size_action),
             ('reset-app-font-size', self._on_app_font_size_action),
             ('toggle-chat-list', self._on_action),
+            ('toggle-menu-bar', self._on_action),
             ('preview-download', self._on_preview_action),
             ('preview-open', self._on_preview_action),
             ('preview-save-as', self._on_preview_action),
@@ -427,7 +474,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('preview-copy-link', self._on_preview_action),
             ('preview-open-link', self._on_preview_action),
             ('copy-message', self._on_copy_message),
-            ('retract-message', self._on_retract_message),
+            ('moderate-message', self._on_moderate_message),
             ('delete-message-locally', self._on_delete_message_locally),
             ('add-workspace', self._add_workspace),
             ('edit-workspace', self._edit_workspace),
@@ -509,6 +556,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         elif action_name == 'toggle-chat-list':
             self._toggle_chat_list()
 
+        elif action_name == 'toggle-menu-bar':
+            show_menu_bar = not self.get_show_menubar()
+            app.settings.set_app_setting('show_main_menu', show_menu_bar)
+            self.set_show_menubar(show_menu_bar)
+
         return None
 
     def _on_app_font_size_action(
@@ -595,7 +647,16 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                         transient_for=self)
                     return
 
-                shutil.copyfile(preview.orig_path, target_path)
+                try:
+                    shutil.copyfile(preview.orig_path, target_path)
+                except PermissionError as e:
+                    ErrorDialog(
+                        _('Could not save file'),
+                        _('You do not have permissions for this directory.\n'
+                          'Error: %s.') % e,
+                        transient_for=self)
+                    return
+
                 app.settings.set('last_save_dir', str(target_path.parent))
 
             if not preview.orig_exists:
@@ -671,37 +732,43 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         clip.set_text(param.get_string(), -1)
 
     @actionmethod
-    def _on_retract_message(self,
+    def _on_moderate_message(self,
                             _action: Gio.SimpleAction,
-                            params: RetractMessageParam
+                            params: ModerateMessageParam
                             ) -> None:
 
-        def _on_retract(reason: str) -> None:
+        def _on_moderate(reason: str) -> None:
             client = app.get_client(params.account)
-            client.get_module('MUC').retract_message(
-                params.jid, params.stanza_id, reason or None)
+            client.get_module('MUC').moderate_message(
+                params.namespace,
+                params.jid,
+                params.stanza_id,
+                reason or None
+            )
 
         InputDialog(
-            _('Retract Message'),
-            _('Retract message?'),
-            _('Why do you want to retract this message?'),
+            _('Moderate Message'),
+            _('Moderate message?'),
+            _('Why do you want to moderate this message?'),
             [DialogButton.make('Cancel'),
              DialogButton.make('Remove',
-                               text=_('_Retract'),
-                               callback=_on_retract)],
+                               text=_('_Moderate'),
+                               callback=_on_moderate)],
             input_str=_('Spam'),
             transient_for=app.window).show()
 
+    @actionmethod
     def _on_delete_message_locally(self,
                                    _action: Gio.SimpleAction,
-                                   param: GLib.Variant
+                                   params: DeleteMessageParam
                                    ) -> None:
 
         def _on_delete() -> None:
-            log_line_id = param.get_uint32()
-            app.storage.archive.delete_message_from_logs(log_line_id)
-            control = self.get_control()
-            control.remove_message(log_line_id)
+            app.storage.archive.delete_message(params.pk)
+            app.ged.raise_event(
+                events.MessageDeleted(account=params.account,
+                                      jid=params.jid,
+                                      pk=params.pk))
 
         ConfirmationDialog(
             _('Delete Message'),
@@ -1138,18 +1205,19 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             # Read marker must be sent only once
             return
 
-        last_message = app.storage.archive.get_last_conversation_line(
+        last_message = app.storage.archive.get_last_conversation_row(
             account, jid)
-        if last_message is None:
+        if last_message is None or last_message.id is None:
             return
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
         assert isinstance(
             contact, BareContact | GroupchatContact | GroupchatParticipant)
+
         client.get_module('ChatMarkers').send_displayed_marker(
             contact,
-            last_message.message_id,
+            last_message.id,
             last_message.stanza_id)
 
     def _on_window_active(self,
@@ -1234,9 +1302,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         open_window('ContactInfo', account=account, contact=contact)
 
     @staticmethod
-    def execute_command(account: str, jid: str) -> None:
-        # TODO: Resource?
-        open_window('AdHocCommands', account=account, jid=jid)
+    def execute_command(account: str, jids: list[str]) -> None:
+        open_window('AdHocCommands', account=account, jids=jids)
 
     def block_contact(self, account: str, jid: str) -> None:
         client = app.get_client(account)
@@ -1310,27 +1377,24 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._set_startup_finished()
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
-        if not self.chat_exists(event.account, event.jid):
-            if not event.properties.body:
-                # Don’t open control on chatstate etc.
-                return
+        if self.chat_exists(event.account, event.jid):
+            return
 
-            if event.properties.is_muc_pm:
-                self.add_private_chat(event.account,
-                                      event.properties.jid)
+        if event.m_type == MessageType.PM:
+            self.add_private_chat(event.account,
+                                  event.jid)
 
-            else:
-                jid = event.properties.jid.new_as_bare()
-                self.add_chat(event.account, jid, 'contact')
+        else:
+            self.add_chat(event.account, event.jid, 'contact')
 
     def _on_read_state_sync(self, event: events.ReadStateSync) -> None:
-        last_message = app.storage.archive.get_last_conversation_line(
+        last_message = app.storage.archive.get_last_conversation_row(
             event.account, event.jid)
 
         if last_message is None:
             return
 
-        if event.marker_id not in (last_message.message_id,
+        if event.marker_id not in (last_message.id,
                                    last_message.stanza_id):
             return
 
