@@ -1,19 +1,8 @@
-
 # Copyright (C) 2009-2014 Yann Leboulanger <asterix AT lagaule.org>
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -36,7 +25,7 @@ from gajim.common.const import URIType
 from gajim.common.const import XmppUriQuery
 from gajim.common.helpers import filesystem_path_from_uri
 from gajim.common.helpers import is_affiliation_change_allowed
-from gajim.common.helpers import is_retraction_allowed
+from gajim.common.helpers import is_moderation_allowed
 from gajim.common.helpers import is_role_change_allowed
 from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import _
@@ -47,6 +36,7 @@ from gajim.common.modules.contacts import can_add_to_roster
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.preview import Preview
+from gajim.common.storage.archive.const import MessageState
 from gajim.common.structs import URI
 from gajim.common.util.text import escape_iri_path_segment
 
@@ -54,9 +44,10 @@ from gajim.gtk.const import MuteState
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import AddChatActionParams
 from gajim.gtk.structs import ChatListEntryParam
+from gajim.gtk.structs import DeleteMessageParam
+from gajim.gtk.structs import ModerateMessageParam
 from gajim.gtk.structs import MuteContactParam
 from gajim.gtk.structs import RemoveHistoryActionParams
-from gajim.gtk.structs import RetractMessageParam
 from gajim.gtk.util import GajimMenu
 from gajim.gtk.util import MenuItemListT
 
@@ -97,6 +88,16 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
         params = AccountJidParam(account=account, jid=contact.jid)
         menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
 
+    jids = [str(c.jid) for c in contact.get_resources()]
+    if not jids:
+        jids = [contact.jid.bare]
+
+    menu.add_item(
+        _('Execute Command…'),
+        f'app.{account}-execute-command',
+        GLib.Variant.new_strv(jids),
+    )
+
     return menu
 
 
@@ -138,6 +139,9 @@ def get_account_menu(account: str) -> GajimMenu:
 
     client = app.get_client(account)
     server_jid = client.get_own_jid().domain
+    assert server_jid is not None
+
+    server_jid = GLib.Variant.new_strv([server_jid])
 
     menuitems: MenuItemListT = [
         (_('Profile'), f'app.{account}-profile', account),
@@ -182,9 +186,9 @@ def build_accounts_menu() -> None:
         return
 
     if len(accounts_list) > 1:
-        modify_accounts_item = Gio.MenuItem.new(_('_Modify Accounts…'),
+        manage_accounts_item = Gio.MenuItem.new(_('_Manage Accounts…'),
                                                 'app.accounts::')
-        acc_menu.append_item(modify_accounts_item)
+        acc_menu.append_item(manage_accounts_item)
         add_contact_item = Gio.MenuItem.new(_('Add _Contact…'),
                                             'app.add-contact::')
         acc_menu.append_item(add_contact_item)
@@ -193,9 +197,9 @@ def build_accounts_menu() -> None:
             acc_menu.append_submenu(label, get_account_menu(acc))
     else:
         acc_menu = get_account_menu(accounts_list[0])
-        modify_account_item = Gio.MenuItem.new(_('_Modify Account…'),
+        manage_account_item = Gio.MenuItem.new(_('_Manage Account…'),
                                                'app.accounts::')
-        acc_menu.insert_item(0, modify_account_item)
+        acc_menu.insert_item(0, manage_account_item)
         add_contact_item = Gio.MenuItem.new(_('Add _Contact…'),
                                             'app.add-contact::')
         acc_menu.insert_item(1, add_contact_item)
@@ -407,17 +411,27 @@ def populate_uri_context_menu(menu: Gtk.Menu, account: str, uri: URI) -> None:
         menu.append(menuitem)
 
 
-def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
+def get_roster_menu(
+    account: str,
+    contact: BareContact,
+    gateway: bool = False
+) -> GajimMenu:
 
     block_label = _('Block…')
-    if jid_is_blocked(account, jid):
+    if jid_is_blocked(account, str(contact.jid)):
         block_label = _('Unblock')
 
-    value = str(jid)
+    value = str(contact.jid)
+
+    jids = [str(c.jid) for c in contact.get_resources()]
+    if not jids:
+        jids = [contact.jid.bare]
+
+    jids_v = GLib.Variant.new_strv(jids)
 
     menuitems: MenuItemListT = [
         (_('Details'), f'app.{account}-contact-info', value),
-        (_('Execute Command…'), f'app.{account}-execute-command', value),
+        (_('Execute Command…'), f'app.{account}-execute-command', jids_v),
         (block_label, f'app.{account}-block-contact', value),
         (_('Remove…'), f'app.{account}-remove-contact', value),
     ]
@@ -677,41 +691,54 @@ def get_chat_row_menu(contact: types.ChatContactT,
                       timestamp: datetime,
                       message_id: str | None,
                       stanza_id: str | None,
-                      log_line_id: int | None
+                      pk: int | None,
+                      corrected_pk: int | None,
+                      state: MessageState,
+                      is_moderated: bool
                       ) -> GajimMenu:
 
     menu_items: MenuItemListT = []
 
-    format_string = app.settings.get('date_time_format')
-    timestamp_formatted = timestamp.strftime(format_string)
+    # Text can be an empty string,
+    # e.g. if a preview has not been loaded yet
+    if text:
+        timestamp_formatted = timestamp.strftime(
+            app.settings.get('date_time_format'))
 
-    copy_text = f'{timestamp_formatted} - {name}: '
-    if text.startswith(('```', '> ')):
-        # Prepend a line break in order to keep code block/quotes rendering
-        copy_text += '\n'
-    copy_text += text
+        copy_text = f'{timestamp_formatted} - {name}: '
+        if text.startswith(('```', '> ')):
+            # Prepend a line break in order to keep code block/quotes rendering
+            copy_text += '\n'
+        copy_text += text
 
-    menu_items.append(
-        (p_('Message row action', 'Copy'),
-         'win.copy-message',
-         copy_text))
+        menu_items.append(
+            (p_('Message row action', 'Copy'),
+            'win.copy-message',
+            copy_text))
 
-    menu_items.append(
-        (p_('Message row action', 'Select Messages…'),
-         'win.activate-message-selection',
-         GLib.Variant('u', log_line_id or 0)))
+        show_quote = True
+        if isinstance(contact, GroupchatContact):
+            if contact.is_joined and state == MessageState.ACKNOWLEDGED:
+                self_contact = contact.get_self()
+                assert self_contact is not None
+                show_quote = not self_contact.role.is_visitor
+            else:
+                show_quote = False
+
+        show_reply = True
+        if isinstance(contact, GroupchatContact):
+            if contact.is_joined and stanza_id is not None:
+                self_contact = contact.get_self()
+                assert self_contact is not None
+                show_reply = not self_contact.role.is_visitor
+            else:
+                show_reply = False
+
+        if not show_reply and show_quote:
+            # Use XEP-0393 quotes if XEP-0461 message reply is not available
+            menu_items.append((
+                p_('Message row action', 'Quote…'), 'win.quote', text))
 
-    show_quote = True
-    if isinstance(contact, GroupchatContact):
-        if contact.is_joined:
-            self_contact = contact.get_self()
-            assert self_contact is not None
-            show_quote = not self_contact.role.is_visitor
-        else:
-            show_quote = False
-    if show_quote:
-        menu_items.append((
-            p_('Message row action', 'Quote…'), 'win.quote', text))
 
     show_correction = False
     if message_id is not None:
@@ -721,33 +748,51 @@ def get_chat_row_menu(contact: types.ChatContactT,
         menu_items.append((
             p_('Message row action', 'Correct…'), 'win.correct-message', None))
 
-    show_retract = False
     if isinstance(contact, GroupchatContact) and contact.is_joined:
         resource_contact = contact.get_resource(name)
         self_contact = contact.get_self()
         assert self_contact is not None
-        is_allowed = is_retraction_allowed(self_contact, resource_contact)
+        is_allowed = is_moderation_allowed(self_contact, resource_contact)
 
         disco_info = app.storage.cache.get_last_disco_info(contact.jid)
         assert disco_info is not None
 
-        if disco_info.has_message_moderation and is_allowed:
-            show_retract = True
-    if show_retract and stanza_id is not None:
-        param = RetractMessageParam(
+        if (
+            disco_info.has_message_moderation and
+            stanza_id is not None and
+            state == MessageState.ACKNOWLEDGED and
+            not is_moderated and
+            is_allowed
+        ):
+
+            ns = disco_info.moderation_namespace
+            assert ns is not None
+            param = ModerateMessageParam(
+                account=contact.account,
+                jid=contact.jid,
+                stanza_id=stanza_id,
+                namespace=ns)
+            menu_items.append((
+                p_('Message row action', 'Moderate…'),
+                'win.moderate-message',
+                param))
+
+    menu_items.append(
+        (p_('Message row action', 'Select Messages…'),
+         'win.activate-message-selection',
+         GLib.Variant('u', corrected_pk or 0)))
+
+    if pk is not None and state == MessageState.ACKNOWLEDGED:
+        param = DeleteMessageParam(
             account=contact.account,
             jid=contact.jid,
-            stanza_id=stanza_id)
-        menu_items.append((
-            p_('Message row action', 'Retract…'),
-            'win.retract-message',
-            param))
+            pk=pk)
 
-    if log_line_id is not None:
         menu_items.append(
-            (p_('Message row action', 'Delete Message Locally…'),
-            'win.delete-message-locally',
-            GLib.Variant('u', log_line_id or 0)))
+            (p_('Message row action',
+             'Delete Message Locally…'),
+             'win.delete-message-locally',
+              param))
 
     return GajimMenu.from_list(menu_items)
 
