@@ -12,26 +12,19 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 
+import hashlib
 import logging
 import sys
 import textwrap
+from pathlib import Path
 
+import cairo
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Gio
@@ -40,6 +33,7 @@ from gi.repository import Gtk
 from nbxmpp.protocol import JID
 
 from gajim.common import app
+from gajim.common import configpaths
 from gajim.common import events
 from gajim.common import ged
 from gajim.common.client import Client
@@ -50,13 +44,13 @@ from gajim.common.ged import EventHelper
 from gajim.common.helpers import allow_showing_notification
 from gajim.common.helpers import play_sound
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import ResourceContact
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import OpenEventActionParams
 from gajim.gtk.util import add_css_to_widget
-from gajim.gtk.util import get_monitor_scale_factor
 from gajim.gtk.util import get_total_screen_geometry
 
 log = logging.getLogger('gajim.gtk.notification')
@@ -147,7 +141,7 @@ class Windows(NotificationBackend):
 
     def _send(self, event: events.Notification) -> None:
         timeout = app.settings.get('notification_timeout')
-        self._withdraw()
+        self._withdraw([])
         self._active_notification = PopupNotification(event, timeout)
 
         def _on_popup_destroy(_widget: Gtk.Window) -> None:
@@ -155,7 +149,7 @@ class Windows(NotificationBackend):
 
         self._active_notification.connect('destroy', _on_popup_destroy)
 
-    def _withdraw(self, *args: Any) -> None:
+    def _withdraw(self, details: list[Any]) -> None:
         if self._active_notification is not None:
             self._active_notification.destroy()
 
@@ -192,8 +186,7 @@ class PopupNotification(Gtk.Window):
 
         if event.type == 'incoming-message':
             assert event.jid is not None
-            pixbuf = _get_avatar_for_notification(
-                event.account, event.jid)
+            pixbuf = _get_pixbuf_icon(event.account, event.jid)
             self._ui.image.set_from_pixbuf(pixbuf)
         else:
             icon_name = self._get_icon_name(event)
@@ -290,20 +283,23 @@ class Linux(NotificationBackend):
 
     def __init__(self):
         NotificationBackend.__init__(self)
-        self._dbus_available: bool = False
+        self._notifications_supported: bool = False
         self._caps: list[str] = []
         self._detect_dbus_caps()
         log.info('Detected notification capabilities: %s', self._caps)
 
     def _detect_dbus_caps(self) -> None:
-        if app.desktop_env == 'gnome':
+        if app.is_flatpak() or app.desktop_env == 'gnome':
             # Gnome Desktop does not use org.freedesktop.Notifications.
             # It has its own API at org.gtk.Notifications, which is not an
             # implementation of the freedesktop spec. There is no documentation
             # on what it currently supports, we can assume at least what the
             # GLib.Notification API offers (icons, actions).
+            #
+            # If the app is run as flatpak the portal API is used
+            # https://flatpak.github.io/xdg-desktop-portal/docs
             self._caps = ['actions']
-            self._dbus_available = True
+            self._notifications_supported = True
             return
 
         def on_proxy_ready(_source: Gio.DBusProxy,
@@ -314,7 +310,7 @@ class Linux(NotificationBackend):
             except GLib.Error as error:
                 log.warning('Notifications D-Bus not available: %s', error)
             else:
-                self._dbus_available = True
+                self._notifications_supported = True
                 log.info('Notifications D-Bus connected')
 
         log.info('Connecting to Notifications D-Bus')
@@ -328,7 +324,7 @@ class Linux(NotificationBackend):
                                   on_proxy_ready)
 
     def _send(self, event: events.Notification) -> None:
-        if not self._dbus_available:
+        if not self._notifications_supported:
             return
 
         notification = Gio.Notification()
@@ -393,9 +389,13 @@ class Linux(NotificationBackend):
         return None
 
     def _make_icon(self, event: events.Notification) -> Gio.Icon:
-        if (event.type == 'incoming-message' and app.desktop_env == 'gnome'):
+        if event.type == 'incoming-message':
             assert event.jid is not None
-            return _get_avatar_for_notification(event.account, event.jid)
+
+            if app.is_flatpak():
+                return _get_bytes_icon(event.account, event.jid)
+
+            return _get_file_icon(event.account, event.jid)
 
         if event.icon_name is not None:
             return Gio.ThemedIcon.new(event.icon_name)
@@ -405,7 +405,7 @@ class Linux(NotificationBackend):
         return Gio.ThemedIcon.new(icon_name)
 
     def _withdraw(self, details: list[Any]) -> None:
-        if not self._dbus_available:
+        if not self._notifications_supported:
             return
         notification_id = self._make_id(details)
 
@@ -417,23 +417,50 @@ class Linux(NotificationBackend):
         return ','.join(map(str, details))
 
 
-def _get_avatar_for_notification(account: str,
-                                 jid: JID | str) -> GdkPixbuf.Pixbuf:
-    scale = get_monitor_scale_factor()
+def _get_surface_for_notification(account: str,
+                                  jid: JID | str) -> cairo.ImageSurface:
     size = AvatarSize.NOTIFICATION
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
+    if isinstance(contact, GroupchatContact):
+        return contact.get_avatar(size, 1)
+
     assert not isinstance(contact, ResourceContact)
-    surface = contact.get_avatar(size, scale)
+    return contact.get_avatar(size, 1, add_show=False)
+
+
+def _get_pixbuf_icon(account: str, jid: JID | str) -> GdkPixbuf.Pixbuf:
+    size = AvatarSize.NOTIFICATION
+    surface = _get_surface_for_notification(account, jid)
     pixbuf = Gdk.pixbuf_get_from_surface(surface,
                                          0,
                                          0,
-                                         size * scale,
-                                         size * scale)
+                                         size,
+                                         size)
     assert pixbuf is not None
     return pixbuf
 
 
+def _get_bytes_icon(account: str, jid: JID | str) -> Gio.BytesIcon:
+    pixbuf = _get_pixbuf_icon(account, jid)
+    _, data = pixbuf.save_to_bufferv('png')
+    return Gio.BytesIcon(bytes=GLib.Bytes.new(data))
+
+
+def _get_file_icon(account: str, jid: JID | str) -> Gio.FileIcon:
+    surface = _get_surface_for_notification(account, jid)
+    path = _get_path_for_icon(surface)
+    return Gio.FileIcon(file=Gio.File.new_for_path(str(path)))
+
+
+def _get_path_for_icon(surface: cairo.ImageSurface) -> Path:
+    path = configpaths.get('AVATAR_ICONS') / hashlib.sha1(
+        bytes(surface.get_data())).hexdigest()
+    if not path.exists():
+        surface.write_to_png(str(path))
+    return path
+
+
 def get_notification_backend() -> NotificationBackend:
     if sys.platform == 'win32':
         return Windows()
