@@ -3,17 +3,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -23,6 +13,7 @@ import logging
 import math
 import sys
 import textwrap
+from datetime import datetime
 from functools import lru_cache
 from functools import wraps
 from importlib import import_module
@@ -48,13 +39,20 @@ from PIL import UnidentifiedImageError
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import types
+from gajim.common.const import AvatarSize
 from gajim.common.const import Display
 from gajim.common.const import LOCATION_DATA
 from gajim.common.const import StyleAttr
 from gajim.common.ged import EventHelper as CommonEventHelper
+from gajim.common.helpers import format_idle_time
 from gajim.common.helpers import URL_REGEX
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
 from gajim.common.structs import VariantMixin
 from gajim.common.styling import PlainBlock
 
@@ -112,7 +110,20 @@ def load_icon_surface(
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_surface(None)
+
+    try:
+        surface = icon_info.load_surface(None)
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return surface
 
 
 def load_icon_pixbuf(icon_name: str,
@@ -124,7 +135,20 @@ def load_icon_pixbuf(icon_name: str,
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_icon()
+
+    try:
+        icon = icon_info.load_icon()
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return icon
 
 
 def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
@@ -256,9 +280,13 @@ def get_completion_liststore(entry: Gtk.Entry) -> Gtk.ListStore:
 def get_cursor(name: str) -> Gdk.Cursor:
     display = Gdk.Display.get_default()
     assert display is not None
-    cursor = Gdk.Cursor.new_from_name(display, name)
-    if cursor is not None:
-        return cursor
+    try:
+        cursor = Gdk.Cursor.new_from_name(display, name)
+        if cursor is not None:
+            return cursor
+    except TypeError as e:
+        log.exception(e)
+
     cursor = Gdk.Cursor.new_from_name(display, 'default')
     assert cursor is not None
     return cursor
@@ -352,6 +380,7 @@ def make_rgba(color_string: str) -> Gdk.RGBA:
 
 
 def get_monitor_scale_factor() -> int:
+    # Does not work on Wayland
     display = Gdk.Display.get_default()
     assert display is not None
     monitor = display.get_primary_monitor()
@@ -478,6 +507,13 @@ class MultiLineLabel(Gtk.Label):
         self.set_single_line_mode(False)
         self.set_selectable(True)
 
+    # make sure full width is used when fitting into GtkGrid
+    def do_get_preferred_width(self):
+        layout = self.get_layout()
+        layout.set_width(-1)
+        width, _ = layout.get_pixel_size()
+        return width, width
+
 
 class MaxWidthComboBoxText(Gtk.ComboBoxText):
     def __init__(self, *args: Any, **kwargs: Any) -> None:
@@ -548,6 +584,59 @@ def add_css_to_widget(widget: Any, css: str) -> None:
                          Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
 
+def get_contact_name_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT
+    ) -> str:
+
+    if isinstance(contact, BareContact) and contact.is_self:
+        return _('Me')
+
+    if db_row.type == MessageType.CHAT:
+        if db_row.direction == ChatDirection.INCOMING:
+            return contact.name
+        return app.nicks[contact.account]
+
+    elif db_row.type == MessageType.GROUPCHAT:
+        resource = db_row.resource
+        if resource is None:
+            # Fall back to MUC name if contact name is None
+            # (may be the case for service messages from the MUC)
+            return contact.name
+        return resource
+
+    elif db_row.type == MessageType.PM:
+        resource = db_row.resource
+        assert resource is not None
+        return resource
+
+    else:
+        raise ValueError
+
+
+def get_avatar_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT,
+        scale: int,
+        size: AvatarSize
+    ) -> cairo.ImageSurface | None:
+
+    if isinstance(contact, GroupchatContact):
+        name = get_contact_name_for_message(db_row, contact)
+        resource_contact = contact.get_resource(name)
+        return resource_contact.get_avatar(
+            size, scale, add_show=False)
+
+    if db_row.direction == ChatDirection.OUTGOING:
+        client = app.get_client(contact.account)
+        self_contact = client.get_module('Contacts').get_contact(
+            client.get_own_jid().bare)
+        assert isinstance(self_contact, BareContact)
+        return self_contact.get_avatar(size, scale, add_show=False)
+
+    return contact.get_avatar(size, scale, add_show=False)
+
+
 def get_pixbuf_from_data(file_data: bytes) -> GdkPixbuf.Pixbuf | None:
     # TODO: This already exists in preview_helpery pixbuf_from_data
     '''
@@ -564,7 +653,7 @@ def get_pixbuf_from_data(file_data: bytes) -> GdkPixbuf.Pixbuf | None:
         log.warning('loading avatar using pixbufloader failed, trying to '
                     'convert avatar image using pillow')
         try:
-            avatar = Image.open(BytesIO(file_data)).convert('RGBA')
+            avatar = Image.open(BytesIO(file_data)).convert('RGBA')  # pyright: ignore
             array = GLib.Bytes.new(avatar.tobytes())  # pyright: ignore
             width, height = avatar.size
             pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
@@ -618,7 +707,7 @@ def load_pixbuf(path: str | Path,
     except GLib.Error:
         try:
             with open(path, 'rb') as im_handle:
-                img = Image.open(im_handle)
+                img = Image.open(im_handle)  # pyright: ignore
                 avatar = img.convert('RGBA')
         except (NameError, OSError, UnidentifiedImageError):
             log.warning('Pillow convert failed: %s', path)
@@ -794,6 +883,46 @@ def wrap_with_event_box(klass: Any) -> Any:
     return klass_wrapper
 
 
+class GroupBadge(Gtk.Label):
+    def __init__(self, group: str) -> None:
+        Gtk.Label.__init__(
+            self,
+            ellipsize=Pango.EllipsizeMode.END,
+            no_show_all=True,
+            halign=Gtk.Align.END,
+            valign=Gtk.Align.CENTER,
+            hexpand=True,
+            tooltip_text=group,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('group')
+        self.set_text(group)
+        self.show()
+
+
+class IdleBadge(Gtk.Label):
+    def __init__(self, idle: datetime | None = None) -> None:
+        Gtk.Label.__init__(
+            self,
+            halign=Gtk.Align.START,
+            hexpand=True,
+            ellipsize=Pango.EllipsizeMode.NONE,
+            no_show_all=True,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('dim-label')
+        self.get_style_context().add_class('small-label')
+        if idle is not None:
+            self.set_idle(idle)
+
+        self.show()
+
+    def set_idle(self, idle: datetime) -> None:
+        self.set_text(_('Last seen: %s') % format_idle_time(idle))
+        format_string = app.settings.get('date_time_format')
+        self.set_tooltip_text(idle.strftime(format_string))
+
+
 class AccountBadge(Gtk.Label):
     def __init__(self, account: str | None = None) -> None:
         Gtk.Label.__init__(self)
@@ -853,7 +982,7 @@ _grapheme_buffer = Gtk.TextBuffer()
 def get_first_graphemes(text: str, n: int) -> str:
     # This should be possible with lower-level APIs like Pango.break_* or
     # Pango.get_log_attrs, but their Python bindings seem totally broken.
-    # The re-use of one global buffer is to mitigate very probable memory leaks.
+    # The reuse of one global buffer is to mitigate very probable memory leaks.
     _grapheme_buffer.set_text(text)
     cursor = _grapheme_buffer.get_start_iter()
     cursor.forward_cursor_positions(n)
