@@ -2,25 +2,14 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 import os
 import platform
 import signal
 import sqlite3
 import sys
+import warnings
 from ctypes import byref
 from ctypes import CDLL
 from ctypes import create_string_buffer
@@ -29,13 +18,13 @@ from types import FrameType
 
 from packaging.version import Version as V
 
-_MIN_NBXMPP_VER = '4.5.3'
+_MIN_NBXMPP_VER = '5.0.0'
 _MIN_GTK_VER = '3.24.30'
 _MIN_CAIRO_VER = '1.16.0'
 _MIN_PYGOBJECT_VER = '3.42.0'
 _MIN_GLIB_VER = '2.66.0'
 _MIN_PANGO_VER = '1.50.0'
-_MIN_SQLITE_VER = '3.33.0'
+_MIN_SQLITE_VER = '3.35.0'
 
 
 def check_version(dep_name: str, current_ver: str, min_ver: str) -> None:
@@ -138,6 +127,9 @@ def _run_app() -> None:
     if sys.platform != 'win32':
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
+    # https://gitlab.gnome.org/GNOME/pygobject/-/issues/12
+    warnings.filterwarnings('ignore', '.*g_value_get_int.*G_VALUE_HOLDS_INT', Warning)
+
     application.run(sys.argv)
 
 
