@@ -2,9 +2,7 @@
 # Copyright 2016 Christoph Reiter
 # Copyright 2017 Philipp Hörist
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation.
+# SPDX-License-Identifier: GPL-2.0-only
 
 set -e
 DIR="$( cd "$( dirname "$0" )" && pwd )"
@@ -82,7 +80,7 @@ function install_deps {
         "${MINGW_PACKAGE_PREFIX}"-python-setuptools \
         "${MINGW_PACKAGE_PREFIX}"-python-setuptools-scm \
         "${MINGW_PACKAGE_PREFIX}"-python-six \
-        "${MINGW_PACKAGE_PREFIX}"-python-winsdk \
+        "${MINGW_PACKAGE_PREFIX}"-python-sqlalchemy \
         "${MINGW_PACKAGE_PREFIX}"-gtk3 \
         "${MINGW_PACKAGE_PREFIX}"-gtksourceview4 \
         "${MINGW_PACKAGE_PREFIX}"-gstreamer \
@@ -103,17 +101,29 @@ function install_deps {
         "${MINGW_PACKAGE_PREFIX}"-webp-pixbuf-loader \
         "${MINGW_PACKAGE_PREFIX}"-sqlite3
 
+    # Temporarily downgrade gobject-introspection, see:
+    # https://github.com/msys2/MINGW-packages/issues/21044
+    build_pacman --noconfirm -U \
+        https://repo.msys2.org/mingw/mingw64/"${MINGW_PACKAGE_PREFIX}"-glib2-2.80.0-1-any.pkg.tar.zst \
+        https://repo.msys2.org/mingw/mingw64/"${MINGW_PACKAGE_PREFIX}"-gobject-introspection-runtime-1.78.1-1-any.pkg.tar.zst \
+        https://repo.msys2.org/mingw/mingw64/"${MINGW_PACKAGE_PREFIX}"-gobject-introspection-1.78.1-1-any.pkg.tar.zst
+
     PIP_REQUIREMENTS="\
 git+https://dev.gajim.org/gajim/python-nbxmpp.git
+git+https://dev.gajim.org/gajim/omemo-dr.git
 python-gnupg
 qrcode
 css_parser
 sentry-sdk
+emoji
+winrt-Windows.Foundation
+winrt-Windows.UI
+winrt-Windows.UI.ViewManagement
 "
-
-    build_pip install precis-i18n
-    SETUPTOOLS_USE_DISTUTILS=stdlib build_pip install git+https://dev.gajim.org/gajim/omemo-dr.git
-    build_pip install $(echo "$PIP_REQUIREMENTS" | tr ["\\n"] [" "])
+    # Workaround for https://dev.gajim.org/gajim/gajim/-/issues/11490
+    # Env variable described in https://www.msys2.org/docs/python/
+    SETUPTOOLS_USE_DISTUTILS=stdlib build_pip install precis-i18n
+    SETUPTOOLS_USE_DISTUTILS=stdlib build_pip install $(echo "$PIP_REQUIREMENTS" | tr ["\\n"] [" "])
 
     # remove the large png icons, they should be used rarely and svg works fine
     rm -Rf "${MINGW_ROOT}/share/icons/Adwaita/512x512"
@@ -257,6 +267,8 @@ function cleanup_install {
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstfaac.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstschro.dll
     rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstcacasink.dll
+    # Crashes often on Windows maschines
+    rm -f "${MINGW_ROOT}"/lib/gstreamer-1.0/libgstamfcodec.dll
 
     rm -f "${MINGW_ROOT}"/bin/libBulletCollision.dll
     rm -f "${MINGW_ROOT}"/bin/libBulletDynamics.dll
